LOW = None
HIGH =None
OUT = None
BCM = None

def output(*args, **kwargs):
    ...

def setmode(*args, **kwargs):
    ...

def setwarnings(flag: bool):
    ...

def setup(*args, **kwargs):
    ...

def cleanup():
    ...
